<?php

namespace Newage\Videos\Http\Controllers;

use App\Http\Controllers\ResourceController as BaseController;
use Form;
use Newage\Videos\Http\Requests\VideoRequest;
use Newage\Videos\Interfaces\VideoRepositoryInterface;
use Newage\Videos\Models\Video;

/**
 * Resource controller class for video.
 */
class VideoResourceController extends BaseController
{

    /**
     * Initialize video resource controller.
     *
     * @param type VideoRepositoryInterface $video
     *
     * @return null
     */
    public function __construct(VideoRepositoryInterface $video)
    {
        parent::__construct();
        $this->repository = $video;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Newage\Videos\Repositories\Criteria\VideoResourceCriteria::class);
    }

    /**
     * Display a list of video.
     *
     * @return Response
     */
    public function index(VideoRequest $request)
    {
        $view = $this->response->theme->listView();

        if ($this->response->typeIs('json')) {
            $function = camel_case('get-' . $view);
            return $this->repository
                ->setPresenter(\Newage\Videos\Repositories\Presenter\VideoPresenter::class)
                ->$function();
        }

        $videos = $this->repository->paginate();

        return $this->response->title(trans('videos::video.names'))
            ->view('videos::video.index', true)
            ->data(compact('videos'))
            ->output();
    }

    /**
     * Display video.
     *
     * @param Request $request
     * @param Model   $video
     *
     * @return Response
     */
    public function show(VideoRequest $request, Video $video)
    {

        if ($video->exists) {
            $view = 'videos::video.show';
        } else {
            $view = 'videos::video.new';
        }

        return $this->response->title(trans('app.view') . ' ' . trans('videos::video.name'))
            ->data(compact('video'))
            ->view($view, true)
            ->output();
    }

    /**
     * Show the form for creating a new video.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(VideoRequest $request)
    {

        $video = $this->repository->newInstance([]);
        return $this->response->title(trans('app.new') . ' ' . trans('videos::video.name')) 
            ->view('videos::video.create', true) 
            ->data(compact('video'))
            ->output();
    }

    /**
     * Create new video.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(VideoRequest $request)
    {
        try {
            $attributes              = $request->all();
            $attributes['user_id']   = user_id();
            $attributes['user_type'] = user_type();
            $video                 = $this->repository->create($attributes);

            return $this->response->message(trans('messages.success.created', ['Module' => trans('videos::video.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('videos/video/' . $video->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/videos/video'))
                ->redirect();
        }

    }

    /**
     * Show video for editing.
     *
     * @param Request $request
     * @param Model   $video
     *
     * @return Response
     */
    public function edit(VideoRequest $request, Video $video)
    {
        return $this->response->title(trans('app.edit') . ' ' . trans('videos::video.name'))
            ->view('videos::video.edit', true)
            ->data(compact('video'))
            ->output();
    }

    /**
     * Update the video.
     *
     * @param Request $request
     * @param Model   $video
     *
     * @return Response
     */
    public function update(VideoRequest $request, Video $video)
    {
        try {
            $attributes = $request->all();

            $video->update($attributes);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('videos::video.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('videos/video/' . $video->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('videos/video/' . $video->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the video.
     *
     * @param Model   $video
     *
     * @return Response
     */
    public function destroy(VideoRequest $request, Video $video)
    {
        try {

            $video->delete();
            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('videos::video.name')]))
                ->code(202)
                ->status('success')
                ->url(guard_url('videos/video/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('videos/video/' . $video->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove multiple video.
     *
     * @param Model   $video
     *
     * @return Response
     */
    public function delete(VideoRequest $request, $type)
    {
        try {
            $ids = hashids_decode($request->input('ids'));

            if ($type == 'purge') {
                $this->repository->purge($ids);
            } else {
                $this->repository->delete($ids);
            }

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('videos::video.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('videos/video'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/videos/video'))
                ->redirect();
        }

    }

    /**
     * Restore deleted videos.
     *
     * @param Model   $video
     *
     * @return Response
     */
    public function restore(VideoRequest $request)
    {
        try {
            $ids = hashids_decode($request->input('ids'));
            $this->repository->restore($ids);

            return $this->response->message(trans('messages.success.restore', ['Module' => trans('videos::video.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('/videos/video'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/videos/video/'))
                ->redirect();
        }

    }

}
